"""
Runable interface
"""

from abc import ABC, abstractmethod


class Runable(ABC):  # pylint: disable=too-few-public-methods
    """Interface for a runable object"""

    @abstractmethod
    def run(self, *args, **kwargs) -> None:
        """Abstract method to run an object

        :param args: Arguments
        :type args: Tuple
        :param kwargs: Keyword arguments
        :type kwargs: Dict

        :rtype: None
        :returns: Nothing
        """



from ansible_aap_api_client.schemas import (
    InventoryRequestSchema,
    InventoryHostRequestSchema,
    InventoryGroupRequestSchema,
)


def test_inventory_request_schema():
    obj = InventoryRequestSchema(name="test", description="test", organization=1)


def test_inventory_host_request_schema():
    obj = InventoryHostRequestSchema(name="test", description="test")


def test_inventory_group_request_schema():
    obj = InventoryGroupRequestSchema(name="test", description="test")



import pytest
from ansible_aap_api_client import JobTemplate


def test_get_all_job_templates(requests_get_two_job_template):
    obj = JobTemplate(base_url="https://localhost:5000", username="test", password="test")
    obj.get_all_job_templates()


def test_get_job_template(requests_get_singe_job_template):
    obj = JobTemplate(base_url="https://localhost:5000", username="test", password="test")
    obj.get_job_template(name="Demo Job Template")


def test_get_job_template_bad(requests_get_singe_job_template):
    obj = JobTemplate(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.get_job_template(name=1)


def test_get_job_template_id(requests_get_singe_job_template):
    obj = JobTemplate(base_url="https://localhost:5000", username="test", password="test")
    obj.get_job_template_id(name="Demo Job Template")


def test_get_job_template_id_bad(requests_get_two_job_template):
    obj = JobTemplate(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(ValueError):
        obj.get_job_template_id(name="Demo Job Template")

    with pytest.raises(TypeError):
        obj.get_job_template_id(name=1)


def test_launch_job_template(requests_job_template_job_launch):
    obj = JobTemplate(base_url="https://localhost:5000", username="test", password="test")
    obj.launch_job_template(job_template_id=7)


def test_launch_job_template_bad(requests_job_template_job_launch):
    obj = JobTemplate(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.launch_job_template(job_template_id="bad")



import pytest
from ansible_aap_api_client import Job


def test_get_job(requests_single_job):
    obj = Job(base_url="https://localhost:5000", username="test", password="test")
    obj.get_job(job_id=35)


def test_get_job_bad(requests_single_job):
    obj = Job(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.get_job(job_id="bad")


def test_get_job_stdout(requests_job_stdout):
    obj = Job(base_url="https://localhost:5000", username="test", password="test")
    obj.get_job_stdout(job_id=35, output_format="txt")


def test_get_job_stdout_bad(requests_job_stdout):
    obj = Job(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.get_job_stdout(job_id="bad", output_format="txt")


def test_get_job_status(requests_single_job):
    obj = Job(base_url="https://localhost:5000", username="test", password="test")
    obj.get_job_status(job_id=35)


def test_get_job_status_bad(requests_single_job):
    obj = Job(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.get_job_status(job_id="bad")



"""
AAP API client for organizations
"""

from ansible_aap_api_client.base_connection import _BaseConnection
from ansible_aap_api_client.schemas import OrganizationRequestSchema


class Organization(_BaseConnection):
    """Organization class

    :type base_url: str
    :param base_url: The base url to use
    :type username: str
    :param username: The username to use
    :type password: str
    :param password: The password to use
    :type ssl_verify: Optional[Union[bool, str]] = True
    :param ssl_verify: The SSL verification True or False or a path to a certificate
    """

    organizations_uri = "/organizations/"

    def get_all_organizations(self) -> dict:
        """Get all organizations

        :rtype: Dict
        :returns: Response
        """
        return self._get(uri=self.organizations_uri).json()

    def get_organization(self, name: str) -> dict:
        """Get all instances of an organization by name

        :type name: str
        :param name: The name of the organization

        :rtype: Dict
        :returns: Response

        :raises TypeError: If name is not of type str
        """
        if not isinstance(name, str):
            raise TypeError(f"name must be of type str, but received {type(name)}")

        return self._get(uri=self.organizations_uri, params={"name": name}).json()

    def get_organization_id(self, name: str) -> int:
        """Get the id of an organization if one exists

        :type name: str
        :param name: The name of the organization

        :rtype: int
        :returns: The id of the organization

        :raises ValueError: If zero or more than one instance is found
        :raises TypeError: If name is not of type str
        """
        if not isinstance(name, str):
            raise TypeError(f"name must be of type str, but received {type(name)}")

        response = self.get_organization(name=name).get("results")

        if len(response) != 1:
            raise ValueError(f"found {len(response)} organizations with name {name}")

        return response[0]["id"]

    def delete_organization(self, organization_id: int) -> int:
        """Delete organization

        :type organization_id: int
        :param organization_id: The id of the organization

        :rtype: Integer
        :returns: Response Status Code

        :raises TypeError: If organization_id is not of type int
        """
        uri = f"{self.organizations_uri}{organization_id}/"

        if not isinstance(organization_id, int):
            raise TypeError(f"organization_id must be of type int, but received {type(organization_id)}")

        return self._delete(uri=uri).status_code

    def update_organization(self, organization_id: int, organization: OrganizationRequestSchema) -> dict:
        """Update a organization

        :type organization_id: int
        :param organization_id: The id of the OrganizationRequestSchema
        :type organization: InventoryHostRequestSchema
        :param organization: The organization object

        :rtype: dict
        :returns: The updated host

        :raises TypeError: If organization_id is not of type int
        :raises TypeError: If organization is not of type OrganizationRequestSchema
        """
        uri = f"{self.organizations_uri}{organization_id}/"

        if not isinstance(organization_id, int):
            raise TypeError(f"organization_id must be of type int, but received {type(organization_id)}")

        if not isinstance(organization, OrganizationRequestSchema):
            raise TypeError(
                f"organization must be of type OrganizationRequestSchema, but received {type(organization)}"
            )

        return self._patch(uri=uri, json_data=organization.dict()).json()

    def create_organization(self, organization: OrganizationRequestSchema):
        """Create an organization

        :type organization: OrganizationRequestSchema
        :param organization: The organization object

        :rtype: dict
        :returns: The created organization

        :raises TypeError: If organization is not of type OrganizationRequestSchema
        """
        if not isinstance(organization, OrganizationRequestSchema):
            raise TypeError(
                f"organization must be of type OrganizationRequestSchema, but received {type(organization)}"
            )

        return self._post(uri=self.organizations_uri, json_data=organization.dict()).json()



import pytest
from ansible_aap_api_client import Organization


def test_get_all_organizations(requests_get_two_organization):
    obj = Organization(base_url="https://localhost:5000", username="test", password="test")
    obj.get_all_organizations()


def test_get_organization(requests_get_two_organization):
    obj = Organization(base_url="https://localhost:5000", username="test", password="test")
    obj.get_organization(name="Default")


def test_get_organization_bad(requests_get_two_organization):
    obj = Organization(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.get_organization(name=1)


def test_get_organization_id(requests_get_single_organization):
    obj = Organization(base_url="https://localhost:5000", username="test", password="test")
    obj.get_organization_id(name="Default")


def test_get_organization_id_bad(requests_get_two_organization):
    obj = Organization(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(ValueError):
        obj.get_organization_id(name="Default")

    with pytest.raises(TypeError):
        obj.get_organization_id(name=1)


def test_delete_organization(requests_delete_organization):
    obj = Organization(base_url="https://localhost:5000", username="test", password="test")
    obj.delete_organization(organization_id=1)


def test_delete_organization_bad(requests_delete_organization):
    obj = Organization(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.delete_organization(organization_id="bad")


def test_create_organization(requests_create_organization, organization_request_schema):
    obj = Organization(base_url="https://localhost:5000", username="test", password="test")
    obj.create_organization(organization=organization_request_schema)


def test_create_organization_bad(requests_create_organization, organization_request_schema):
    obj = Organization(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.create_organization(organization="bad")



"""
init for ansible_aap_api_client
"""

from ansible_aap_api_client.ansible_aap_api_client_cli import cli
from ansible_aap_api_client.inventories import Inventory
from ansible_aap_api_client.groups import Group
from ansible_aap_api_client.hosts import Host
from ansible_aap_api_client.organizations import Organization
from ansible_aap_api_client.job_templates import JobTemplate
from ansible_aap_api_client.jobs import Job
from ansible_aap_api_client.inventory_management import InventoryManagement, InventoryBuilder
from ansible_aap_api_client.job_management import JobManagement
from ansible_aap_api_client.schemas import (
    InventoryRequestSchema,
    InventoryHostRequestSchema,
    InventoryGroupRequestSchema,
    OrganizationRequestSchema,
)



"""
AAP Inventories
"""

from ansible_aap_api_client.base_connection import _BaseConnection
from ansible_aap_api_client.schemas import (
    InventoryRequestSchema,
    InventoryHostRequestSchema,
    InventoryGroupRequestSchema,
)


class Inventory(_BaseConnection):
    """Inventory class

    :type base_url: str
    :param base_url: The base url to use
    :type username: str
    :param username: The username to use
    :type password: str
    :param password: The password to use
    :type ssl_verify: Optional[Union[bool, str]] = True
    :param ssl_verify: The SSL verification True or False or a path to a certificate
    """

    inventories_uri = "/inventories/"

    def get_all_inventories(self) -> dict:
        """Get all inventories

        :rtype: Dict
        :returns: Response
        """
        return self._get(uri=self.inventories_uri).json()

    def get_inventory(self, name: str) -> dict:
        """Get all instances of an inventory by name

        :type name: str
        :param name: The inventory name

        :rtype: Dict
        :returns: Response

        :raises TypeError: If name is not of type str
        """
        if not isinstance(name, str):
            raise TypeError(f"name must be of type str, but received {type(name)}")

        return self._get(uri=self.inventories_uri, params={"name": name}).json()

    def get_inventory_id(self, name: str) -> int:
        """Get the id of an inventory if one exists

        :type name: str
        :param name: The name of the inventory

        :rtype: int
        :returns: The id of the inventory

        :raises ValueError: If zero or more than one instance is found
        :raises TypeError: If name is not of type str
        """
        if not isinstance(name, str):
            raise TypeError(f"name must be of type str, but received {type(name)}")

        response = self.get_inventory(name=name).get("results")

        if len(response) != 1:
            raise ValueError(f"found {len(response)} inventories with name {name}")

        return response[0]["id"]

    def delete_inventory(self, inventory_id: int) -> int:
        """Delete inventory

        :type inventory_id: int
        :param inventory_id: The inventory id

        :rtype: Integer
        :returns: Response Status Code

        :raises TypeError: If inventory_id is not of type int
        """
        uri = f"{self.inventories_uri}{inventory_id}/"

        if not isinstance(inventory_id, int):
            raise TypeError(f"inventory_id must be of type int, but received {type(inventory_id)}")

        return self._delete(uri=uri).status_code

    def update_inventory(self, inventory_id: int, inventory: InventoryRequestSchema) -> dict:
        """Update inventory

        :type inventory_id: int
        :param inventory_id: The inventory id
        :type inventory: InventoryRequestSchema
        :param inventory: The inventory data

        :rtype: Dict
        :returns: Response

        :raises TypeError: If inventory is not a InventoryRequestSchema
        :raises TypeError: If inventory_id is not of type int
        """
        uri = f"{self.inventories_uri}{inventory_id}/"

        if not isinstance(inventory_id, int):
            raise TypeError(f"inventory_id must be of type int, but received {type(inventory_id)}")

        if not isinstance(inventory, InventoryRequestSchema):
            raise TypeError(f"inventory must be of type InventoryRequestSchema, but received {type(inventory)}")

        return self._patch(uri=uri, json_data=inventory.dict()).json()

    def create_inventory(self, inventory: InventoryRequestSchema) -> dict:
        """Create inventory

        :type inventory: InventoryRequestSchema
        :param inventory: The inventory to create

        :rtype: Dict
        :returns: Response

        :raises TypeError: If inventory is not a InventoryRequestSchema
        """
        if not isinstance(inventory, InventoryRequestSchema):
            raise TypeError(f"inventory must be of type InventoryRequestSchema, but received {type(inventory)}")

        return self._post(uri=self.inventories_uri, json_data=inventory.dict()).json()

    def add_host_to_inventory(self, inventory_id: int, host: InventoryHostRequestSchema) -> dict:
        """Add host to inventory

        :type inventory_id: int
        :param inventory_id: The inventory id
        :type host: InventoryHostRequestSchema
        :param host: The host to add

        :rtype: Dict
        :returns: Response

        :raises TypeError: If host is not an InventoryHostRequestSchema
        """
        uri = f"{self.inventories_uri}{inventory_id}/hosts/"

        if not isinstance(inventory_id, int):
            raise TypeError(f"inventory_id must be of type int, but received {type(inventory_id)}")

        if not isinstance(host, InventoryHostRequestSchema):
            raise TypeError(f"host must be of type InventoryHostRequestSchema, but received {type(host)}")

        return self._post(uri=uri, json_data=host.dict()).json()

    def add_group_to_inventory(self, inventory_id: int, group: InventoryGroupRequestSchema) -> dict:
        """Add group to inventory

        :type inventory_id: int
        :param inventory_id: The inventory id
        :type group: InventoryGroupRequestSchema
        :param group: The group to add

        :rtype: Dict
        :returns: Response

        :raises TypeError: If group is not an InventoryGroupRequestSchema
        """
        uri = f"{self.inventories_uri}{inventory_id}/groups/"

        if not isinstance(inventory_id, int):
            raise TypeError(f"inventory_id must be of type int, but received {type(inventory_id)}")

        if not isinstance(group, InventoryGroupRequestSchema):
            raise TypeError(f"group must be of type InventoryGroupRequestSchema, but received {type(group)}")

        return self._post(uri=uri, json_data=group.dict()).json()



"""
AAP Job Management
"""

from typing import Optional, Union
import time
from ansible_aap_api_client.inventory_management import InventoryManagement
from ansible_aap_api_client.job_templates import JobTemplate
from ansible_aap_api_client.jobs import Job
from ansible_aap_api_client.interfaces.runable import Runable


class JobManagement(Runable, InventoryManagement, JobTemplate, Job):  # pylint: disable=too-many-ancestors
    """Job management class, to run a job template against an inventory

    :type base_url: str
    :param base_url: The base url to use
    :type username: str
    :param username: The username to use
    :type password: str
    :param password: The password to use
    :type ssl_verify: Union[bool, str]
    :param ssl_verify: The SSL verification True or False or a path to a certificate
    :type job_template_name: str
    :param job_template_name: The name of the job template
    :type inventory_name: str
    :param inventory_name: The name of the inventory

    :raises TypeError: If job_template_name is not of type str
    :raises TypeError: If inventory_name is not of type str
    """

    def __init__(  # pylint: disable=too-many-arguments
        self,
        base_url: str,
        username: str,
        password: str,
        ssl_verify: Union[bool, str],
        job_template_name: str,
        inventory_name: str,
    ) -> None:
        super().__init__(base_url=base_url, username=username, password=password, ssl_verify=ssl_verify)

        if not isinstance(job_template_name, str):
            raise TypeError(f"job_template_name must be of type str, but received {type(job_template_name)}")

        if not isinstance(inventory_name, str):
            raise TypeError(f"inventory_name must be of type str, but received {type(inventory_name)}")

        self.job_template_name = job_template_name
        self.job_template_id = None
        self.inventory_name = inventory_name
        self.inventory_id = None
        self.job_id = None

    def run(self, **kwargs) -> None:
        """Run the job management

        :param kwargs: The keyword arguments to pass to the launch_job_template method

        :rtype: None
        :return: Runs the inventory builder
        """
        self.job_template_id = self.get_job_template_id(name=self.job_template_name)
        self.inventory_id = self.get_inventory_id(name=self.inventory_name)
        self.job_id = self.launch_job_template(
            job_template_id=self.job_template_id, inventory=self.inventory_id, **kwargs
        ).get("id")

    def poll_completion(self, print_status: Optional[bool] = False, **kwargs) -> str:  # pragma: no cover
        """Run the job and poll the completion of a job

        :type print_status: Optional[bool] = False
        :param print_status: Print the status of the job

        :param kwargs: The keyword arguments to pass to the launch_job_template method

        :rtype: String
        :returns: The completed status of the job

        :raises TypeError: If print_status is not of type bool
        """
        if not isinstance(print_status, bool):
            raise TypeError(f"print_status must be of type bool, but received {type(print_status)}")

        if not self.job_id:
            self.run(**kwargs)

        ok_statuses = ["successful", "failed", "error", "cancelled"]

        job_status = "new"

        if print_status:
            print(f"Polling job_id {self.job_id} current status {job_status}")

        while job_status not in ok_statuses:
            time.sleep(5)
            job_status = self.get_job_status(job_id=self.job_id)

            if print_status:
                print(f"Polling job_id {self.job_id} current status {job_status}")

            if job_status in ok_statuses:
                break

        if print_status:
            print(f"Polling job_id {self.job_id} completed status {job_status}")

        return job_status



import pytest
from ansible_aap_api_client import Host


def test_get_all_hosts(requests_get_two_host):
    obj = Host(base_url="https://localhost:5000", username="test", password="test")
    response = obj.get_all_hosts()


def test_get_host(requests_get_two_host):
    obj = Host(base_url="https://localhost:5000", username="test", password="test")
    obj.get_host(name="test_host")


def test_get_host_bad(requests_get_two_host):
    obj = Host(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.get_host(name=1)


def test_get_host_id(requests_get_single_host):
    obj = Host(base_url="https://localhost:5000", username="test", password="test")
    response = obj.get_host_id(name="test_host")

    assert response == 1


def test_get_host_id_bad(requests_get_two_host):
    obj = Host(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(ValueError):
        obj.get_host_id(name="test_host")

    with pytest.raises(TypeError):
        obj.get_host_id(name=1)


def test_delete_host(requests_delete_host):
    obj = Host(base_url="https://localhost:5000", username="test", password="test")
    response = obj.delete_host(host_id=2)


def test_delete_host_bad(requests_delete_host):
    obj = Host(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.delete_host(host_id="bad")


def test_update_host_bad(requests_delete_host):
    obj = Host(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.update_host(host_id="bad", host="bad")

    with pytest.raises(TypeError):
        obj.update_host(host_id=2, host="bad")



"""
A base connection class for the AAP API client.
"""

from typing import Optional, Union
import requests


class _BaseConnection:  # pylint: disable=too-few-public-methods
    """Base connection class

    :type base_url: str
    :param base_url: The base url to use
    :type username: str
    :param username: The username to use
    :type password: str
    :param password: The password to use
    :type ssl_verify: Optional[Union[bool, str]] = True
    :param ssl_verify: The SSL verification True or False or a path to a certificate

    :raises TypeError: If the base_url, username, password is not a string
    :raises TypeError: If ssl_verify is not a bool or str
    """

    def __init__(
        self, base_url: str, username: str, password: str, ssl_verify: Optional[Union[bool, str]] = True
    ) -> None:
        if not isinstance(base_url, str):
            raise TypeError(f"base_url must be of type str, but received {type(base_url)}")

        self.base_url = base_url

        if not isinstance(username, str):
            raise TypeError(f"username must be of type str, but received {type(username)}")

        self.username = username

        if not isinstance(password, str):
            raise TypeError(f"password must be of type str, but received {type(password)}")

        self.password = password

        if isinstance(ssl_verify, str):
            self.ssl_verify = ssl_verify

        elif isinstance(ssl_verify, bool):
            self.ssl_verify = ssl_verify

        else:
            raise TypeError(f"ssl_verify must be of type bool or str, but received {type(ssl_verify)}")

        self.api_version = "/api/v2"
        self.headers = {"Content-Type": "application/json"}

    def _post(self, uri: str, json_data: Optional[dict] = None) -> requests.Response:
        """Protected post method

        :type uri: str
        :param uri: The uri to post
        :type json_data: Optional[dict]
        :param json_data: The JSON data to post

        :rtype: requests.Response
        :return: A response object
        """
        url = f"{self.base_url}{self.api_version}{uri}"
        response = requests.post(
            url=url,
            auth=(self.username, self.password),
            headers=self.headers,
            json=json_data,
            verify=self.ssl_verify,
            timeout=30,
        )

        if not response.ok:  # pragma: no cover
            response.raise_for_status()

        return response

    def _patch(self, uri: str, json_data: Optional[dict] = None) -> requests.Response:
        """Protected patch method

        :type uri: str
        :param uri: The uri to patch
        :type json_data: Optional[dict]
        :param json_data: The JSON data to patch

        :rtype: requests.Response
        :return: A response object
        """
        url = f"{self.base_url}{self.api_version}{uri}"
        response = requests.patch(
            url=url,
            auth=(self.username, self.password),
            headers=self.headers,
            json=json_data,
            verify=self.ssl_verify,
            timeout=30,
        )

        if not response.ok:  # pragma: no cover
            response.raise_for_status()

        return response

    def _get(self, uri: str, params: Optional[dict] = None) -> requests.Response:
        """Protected GET method

        :type uri: str
        :param uri: The URI to use
        :type params: Optional[dict] = None
        :param params: The parameters to use

        :rtype: requests.Response
        :returns: A response object
        """
        url = f"{self.base_url}{self.api_version}{uri}"
        response = requests.get(
            url=url,
            auth=(self.username, self.password),
            params=params,
            headers=self.headers,
            verify=self.ssl_verify,
            timeout=30,
        )

        if not response.ok:  # pragma: no cover
            response.raise_for_status()

        return response

    def _delete(self, uri: str, json_data: Optional[dict] = None) -> requests.Response:
        """Protected delete method

        :type uri: str
        :param uri: The uri to delete
        :type json_data: Optional[dict]
        :param json_data: The JSON data to delete

        :rtype: requests.Response
        :return: A response object
        """
        url = f"{self.base_url}{self.api_version}{uri}"
        response = requests.delete(
            url=url,
            auth=(self.username, self.password),
            headers=self.headers,
            json=json_data,
            verify=self.ssl_verify,
            timeout=30,
        )

        if not response.ok:  # pragma: no cover
            response.raise_for_status()

        return response



"""
AAP Groups
"""

from ansible_aap_api_client.base_connection import _BaseConnection
from ansible_aap_api_client.schemas import InventoryHostRequestSchema


class Host(_BaseConnection):
    """Host class

    :type base_url: str
    :param base_url: The base url to use
    :type username: str
    :param username: The username to use
    :type password: str
    :param password: The password to use
    :type ssl_verify: Optional[Union[bool, str]] = True
    :param ssl_verify: The SSL verification True or False or a path to a certificate
    """

    hosts_uri = "/hosts/"

    def get_all_hosts(self) -> dict:
        """Get all hosts

        :rtype: Dict
        :returns: Response
        """
        return self._get(uri=self.hosts_uri).json()

    def get_host(self, name: str) -> dict:
        """Get all instances of a host by name

        :type name: str
        :param name: The name of the host

        :rtype: Dict
        :returns: Response

        :raises TypeError: If name is not of type str
        """
        if not isinstance(name, str):
            raise TypeError(f"name must be of type str, but received {type(name)}")

        return self._get(uri=self.hosts_uri, params={"name": name}).json()

    def get_host_id(self, name: str) -> int:
        """Get the id of a host if one exists

        :type name: str
        :param name: The name of the host

        :rtype: int
        :returns: The id of the host

        :raises ValueError: If zero or more than one instance is found
        :raises TypeError: If name is not of type str
        """
        if not isinstance(name, str):
            raise TypeError(f"name must be of type str, but received {type(name)}")

        response = self.get_host(name=name).get("results")

        if len(response) != 1:
            raise ValueError(f"Expected 1 result, but received {len(response)}")

        return response[0]["id"]

    def delete_host(self, host_id: int) -> int:
        """Delete a host

        :type host_id: int
        :param host_id: The id of the host

        :rtype: int
        :returns: The id of the deleted host
        """
        uri = f"{self.hosts_uri}{host_id}/"

        if not isinstance(host_id, int):
            raise TypeError(f"host_id must be of type int, but received {type(host_id)}")

        return self._delete(uri=uri).status_code

    def update_host(self, host_id: int, host: InventoryHostRequestSchema) -> dict:
        """Update a host

        :type host_id: int
        :param host_id: The id of the host
        :type host: InventoryHostRequestSchema
        :param host: The host object

        :rtype: dict
        :returns: The updated host
        """
        uri = f"{self.hosts_uri}{host_id}/"

        if not isinstance(host_id, int):
            raise TypeError(f"host_id must be of type int, but received {type(host_id)}")

        if not isinstance(host, InventoryHostRequestSchema):
            raise TypeError(f"host must be of type InventoryHostRequestSchema, but received {type(host)}")

        return self._patch(uri=uri, json_data=host.dict()).json()



"""
AAP API client for inventory management
"""

from typing import List, Union
from ansible_aap_api_client.inventories import Inventory
from ansible_aap_api_client.schemas import (
    InventoryRequestSchema,
    InventoryGroupRequestSchema,
    InventoryHostRequestSchema,
)
from ansible_aap_api_client.hosts import Host
from ansible_aap_api_client.groups import Group
from ansible_aap_api_client.interfaces.runable import Runable


class InventoryManagement(Inventory, Group, Host):
    """Inventory management class

    :type base_url: str
    :param base_url: The base url to use
    :type username: str
    :param username: The username to use
    :type password: str
    :param password: The password to use
    :type ssl_verify: Optional[Union[bool, str]] = True
    :param ssl_verify: The SSL verification True or False or a path to a certificate
    """


class InventoryBuilder(Runable, InventoryManagement):  # pylint: disable=too-many-instance-attributes
    """Inventory builder class, this builds an inventory with groups and hosts

    :type base_url: str
    :param base_url: The base url to use
    :type username: str
    :param username: The username to use
    :type password: str
    :param password: The password to use
    :type ssl_verify: Union[bool, str]
    :param ssl_verify: The SSL verification True or False or a path to a certificate

    :raises TypeError: If inventory is not an instance of InventoryRequestSchema
    """

    IOS_GROUP_VARS = {
        "ansible_connection": "ansible.netcommon.network_cli",
        "ansible_become": True,
        "ansible_become_method": "enable",
        "ansible_network_os": "ios",
    }

    IOSXR_GROUP_VARS = {
        "ansible_connection": "ansible.netcommon.network_cli",
        "ansible_become": True,
        "ansible_become_method": "enable",
        "ansible_network_os": "iosxr",
    }

    NXOS_GROUP_VARS = {
        "ansible_connection": "ansible.netcommon.network_cli",
        "ansible_become": True,
        "ansible_become_method": "enable",
        "ansible_network_os": "nxos",
    }

    EOS_GROUP_VARS = {
        "ansible_connection": "ansible.netcommon.network_cli",
        "ansible_become": True,
        "ansible_become_method": "enable",
        "ansible_network_os": "eos",
    }

    def __init__(  # pylint: disable=too-many-arguments
        self,
        base_url: str,
        username: str,
        password: str,
        ssl_verify: Union[bool, str],
        inventory: InventoryRequestSchema,
    ) -> None:
        super().__init__(base_url, username, password, ssl_verify)

        if not isinstance(inventory, InventoryRequestSchema):
            raise TypeError(f"inventory must be an instance of InventoryRequestSchema but received a {type(inventory)}")

        self.inventory = inventory
        self.inventory_name = inventory.name
        self.inventory_id = None

        self.ios_group_id = None
        self.ios_hosts = []

        self.iosxr_group_id = None
        self.iosxr_hosts = []

        self.nxos_group_id = None
        self.nxos_hosts = []

        self.eos_group_id = None
        self.eos_hosts = []

        self.custom_groups = []
        self.custom_groups_data = {}
        self.custom_hosts = []

    def run(self) -> None:
        """Run the inventory builder

        :rtype: None
        :return: Runs the inventory builder
        """
        self.inventory_id = self.create_inventory(inventory=self.inventory).get("id")

        self.ios_group_id = self.add_group_to_inventory(
            inventory_id=self.inventory_id,
            group=self._get_group_request_schema("ios"),
        ).get("id")

        for host in self.ios_hosts:
            self.add_host_to_group(group_id=self.ios_group_id, host=host)

        self.iosxr_group_id = self.add_group_to_inventory(
            inventory_id=self.inventory_id,
            group=self._get_group_request_schema("iosxr"),
        ).get("id")

        for host in self.iosxr_hosts:
            self.add_host_to_group(group_id=self.iosxr_group_id, host=host)

        self.nxos_group_id = self.add_group_to_inventory(
            inventory_id=self.inventory_id,
            group=self._get_group_request_schema("nxos"),
        ).get("id")

        for host in self.nxos_hosts:
            self.add_host_to_group(group_id=self.nxos_group_id, host=host)

        self.eos_group_id = self.add_group_to_inventory(
            inventory_id=self.inventory_id,
            group=self._get_group_request_schema("eos"),
        ).get("id")

        for host in self.eos_hosts:
            self.add_host_to_group(group_id=self.eos_group_id, host=host)

        for group in self.custom_groups:
            response = self.add_group_to_inventory(inventory_id=self.inventory_id, group=group)
            self.custom_groups_data[group.name] = response.get("id")

        for host in self.custom_hosts:
            self.add_host_to_group(group_id=self.custom_groups_data[host["group_name"]], host=host["host"])

    def _get_group_request_schema(self, nos: str) -> InventoryGroupRequestSchema:
        """Protected method to get group request schema

        :type nos: str
        :param nos: The NOS to use

        :rtype: InventoryGroupRequestSchema
        :return: The group request schema

        :raises ValueError: If the NOS is not supported
        """
        if nos == "ios":
            group_vars = self.IOS_GROUP_VARS
        elif nos == "iosxr":
            group_vars = self.IOSXR_GROUP_VARS
        elif nos == "nxos":
            group_vars = self.NXOS_GROUP_VARS
        elif nos == "eos":
            group_vars = self.EOS_GROUP_VARS
        else:
            raise ValueError(f"Unsupported NOS: {nos}")

        return InventoryGroupRequestSchema(
            name=f"{self.inventory_name}-{nos}",
            description=f"Inventory {self.inventory_name} Group for {nos} devices",
            variables=group_vars,
        )

    def add_ios_host_to_inventory(self, host: InventoryHostRequestSchema) -> None:
        """Add IOS host to inventory

        :type host: InventoryHostRequestSchema
        :param host: The host to add

        :rtype: None
        :return: Adds the host to the inventory

        :raises TypeError: If host is not an instance of InventoryHostRequestSchema
        """
        if not isinstance(host, InventoryHostRequestSchema):
            raise TypeError(f"host must be an instance of InventoryHostRequestSchema but received a {type(host)}")

        self.ios_hosts.append(host)

    def add_ios_hosts_to_inventory(self, hosts: List[InventoryHostRequestSchema]) -> None:
        """Add multiple IOS hosts to inventory

        :type hosts: List[InventoryHostRequestSchema]
        :param hosts: The hosts to add

        :rtype: None
        :return: Adds the hosts to the inventory

        :raises TypeError: If hosts is not a list of InventoryHostRequestSchema
        """
        if not all(isinstance(host, InventoryHostRequestSchema) for host in hosts):
            raise TypeError(f"hosts must be a list of InventoryHostRequestSchema but received a list of {type(hosts)}")

        self.ios_hosts.extend(hosts)

    def add_iosxr_host_to_inventory(self, host: InventoryHostRequestSchema) -> None:
        """Add IOS-XR host to inventory

        :type host: InventoryHostRequestSchema
        :param host: The host to add

        :rtype: None
        :return: Adds the host to the inventory

        :raises TypeError: If host is not an instance of InventoryHostRequestSchema
        """
        if not isinstance(host, InventoryHostRequestSchema):
            raise TypeError(f"host must be an instance of InventoryHostRequestSchema but received a {type(host)}")

        self.iosxr_hosts.append(host)

    def add_iosxr_hosts_to_inventory(self, hosts: List[InventoryHostRequestSchema]) -> None:
        """Add multiple IOS-XR hosts to inventory

        :type hosts: List[InventoryHostRequestSchema]
        :param hosts: The hosts to add

        :rtype: None
        :return: Adds the hosts to the inventory

        :raises TypeError: If hosts is not a list of InventoryHostRequestSchema
        """
        if not all(isinstance(host, InventoryHostRequestSchema) for host in hosts):
            raise TypeError(f"hosts must be a list of InventoryHostRequestSchema but received a list of {type(hosts)}")

        self.iosxr_hosts.extend(hosts)

    def add_nxos_host_to_inventory(self, host: InventoryHostRequestSchema) -> None:
        """Add NX-OS host to inventory

        :type host: InventoryHostRequestSchema
        :param host: The host to add

        :rtype: None
        :return: Adds the host to the inventory

        :raises TypeError: If host is not an instance of InventoryHostRequestSchema
        """
        if not isinstance(host, InventoryHostRequestSchema):
            raise TypeError(f"host must be an instance of InventoryHostRequestSchema but received a {type(host)}")

        self.nxos_hosts.append(host)

    def add_nxos_hosts_to_inventory(self, hosts: List[InventoryHostRequestSchema]) -> None:
        """Add multiple NX-OS hosts to inventory

        :type hosts: List[InventoryHostRequestSchema]
        :param hosts: The hosts to add

        :rtype: None
        :return: Adds the hosts to the inventory

        :raises TypeError: If hosts is not a list of InventoryHostRequestSchema
        """
        if not all(isinstance(host, InventoryHostRequestSchema) for host in hosts):
            raise TypeError(f"hosts must be a list of InventoryHostRequestSchema but received a list of {type(hosts)}")

        self.nxos_hosts.extend(hosts)

    def add_eos_host_to_inventory(self, host: InventoryHostRequestSchema) -> None:
        """Add EOS host to inventory

        :type host: InventoryHostRequestSchema
        :param host: The host to add

        :rtype: None
        :return: Adds the host to the inventory

        :raises TypeError: If host is not an instance of InventoryHostRequestSchema
        """
        if not isinstance(host, InventoryHostRequestSchema):
            raise TypeError(f"host must be an instance of InventoryHostRequestSchema but received a {type(host)}")

        self.eos_hosts.append(host)

    def add_eos_hosts_to_inventory(self, hosts: List[InventoryHostRequestSchema]) -> None:
        """Add multiple EOS hosts to inventory

        :type hosts: List[InventoryHostRequestSchema]
        :param hosts: The hosts to add

        :rtype: None
        :return: Adds the hosts to the inventory

        :raises TypeError: If hosts is not a list of InventoryHostRequestSchema
        """
        if not all(isinstance(host, InventoryHostRequestSchema) for host in hosts):
            raise TypeError(f"hosts must be a list of InventoryHostRequestSchema but received a list of {type(hosts)}")

        self.eos_hosts.extend(hosts)

    def add_custom_group_to_inventory(self, group: InventoryGroupRequestSchema) -> None:
        """Add custom group to inventory

        :type group: InventoryGroupRequestSchema
        :param group: The group to add

        :rtype: None
        :return: Adds the custom group to the inventory

        :raises TypeError: If group is not an instance of InventoryGroupRequestSchema
        """
        if not isinstance(group, InventoryGroupRequestSchema):
            raise TypeError(f"group must be an instance of InventoryGroupRequestSchema but received a {type(group)}")

        self.custom_groups.append(group)

    def add_custom_groups_to_inventory(self, groups: List[InventoryGroupRequestSchema]) -> None:
        """Add custom group to inventory

        :type groups: List[InventoryGroupRequestSchema]
        :param groups: The groups to add

        :rtype: None
        :return: Adds the custom groups to the inventory

        :raises TypeError: If groups is not a list of InventoryGroupRequestSchema
        """
        if not all(isinstance(group, InventoryGroupRequestSchema) for group in groups):
            raise TypeError(
                f"groups must be a list of InventoryGroupRequestSchema but received a list of {type(groups)}"
            )

        self.custom_groups.extend(groups)

    def add_host_to_custom_group_to_inventory(self, group_name: str, host: InventoryHostRequestSchema) -> None:
        """Add host to custom group to inventory

        :type group_name: str
        :param group_name: The group name to add the host to
        :type host: InventoryHostRequestSchema
        :param host: The host to add

        :rtype: None
        :return: Adds the host to the inventory

        :raises TypeError: If group_name is not a string or host is not an instance of InventoryHostRequestSchema
        """
        if not isinstance(group_name, str):
            raise TypeError(f"group_name must be a string but received a {type(group_name)}")

        if not isinstance(host, InventoryHostRequestSchema):
            raise TypeError(f"host must be an instance of InventoryHostRequestSchema but received a {type(host)}")

        self.custom_hosts.append({"group_name": group_name, "host": host})

    def add_hosts_to_custom_group_to_inventory(self, group_name: str, hosts: List[InventoryHostRequestSchema]) -> None:
        """Add hosts to custom group to inventory

        :type group_name: str
        :param group_name: The group name to add the hosts to
        :type hosts: List[InventoryHostRequestSchema]
        :param hosts: The hosts to add

        :rtype: None
        :return: Adds the hosts to the inventory

        :raises TypeError: If group_name is not a string or hosts is not a list of InventoryHostRequestSchema
        """
        if not isinstance(group_name, str):
            raise TypeError(f"group_name must be a string but received a {type(group_name)}")

        if not all(isinstance(host, InventoryHostRequestSchema) for host in hosts):
            raise TypeError(f"hosts must be a list of InventoryHostRequestSchema but received a list of {type(hosts)}")

        for host in hosts:
            self.custom_hosts.append({"group_name": group_name, "host": host})



from ansible_aap_api_client.ansible_aap_api_client_cli import cli_argument_parser


def test_cli_argument_parser_run_job_template():
    arg_parser = cli_argument_parser()
    args = arg_parser.parse_args(
        [
            "-b",
            "https://fake.com",
            "-u",
            "fake-username",
            "-p",
            "fake-password",
            "run-job-template",
            "-t",
            "fake-template",
            "-i",
            "fake-inventory",
        ]
    )

    assert args.which_sub == "run-job-template"
    assert args.base_url == "https://fake.com"
    assert args.username == "fake-username"
    assert args.password == "fake-password"
    assert args.template_name == "fake-template"
    assert args.inventory_name == "fake-inventory"



"""
AAP Jobs
"""

from typing import Optional, Literal
from ansible_aap_api_client.base_connection import _BaseConnection


class Job(_BaseConnection):
    """Job class

    :type base_url: str
    :param base_url: The base url to use
    :type username: str
    :param username: The username to use
    :type password: str
    :param password: The password to use
    :type ssl_verify: Optional[Union[bool, str]] = True
    :param ssl_verify: The SSL verification True or False or a path to a certificate
    """

    jobs_uri = "/jobs/"

    def get_all_jobs(self) -> dict:
        """Get all jobs

        :rtype: Dict
        :returns: Response
        """
        return self._get(uri=self.jobs_uri).json()

    def get_job(self, job_id: int) -> dict:
        """Get a job by id

        :type job_id: int
        :param job_id: The ID of the job

        :rtype: Dict
        :returns: Response

        :raises TypeError: If name is not of type str
        """
        uri = f"{self.jobs_uri}{job_id}/"
        if not isinstance(job_id, int):
            raise TypeError(f"job_id must be of type int, but received {type(job_id)}")

        return self._get(uri=uri).json()

    def get_job_stdout(self, job_id: int, output_format: Optional[Literal["txt", "json", "html"]] = "txt") -> str:
        """Get the stdout of a job by id, this is the ansible run log

        :type job_id: int
        :param job_id: The ID of the job
        :type output_format: Optional[Literal["txt", "json", "html"]] = "txt"
        :param output_format: The format of the output

        :rtype: str
        :returns: The stdout of the job

        :raises TypeError: If name is not of type str
        """
        uri = f"{self.jobs_uri}{job_id}/stdout/"

        if not isinstance(job_id, int):
            raise TypeError(f"job_id must be of type int, but received {type(job_id)}")

        return self._get(uri=uri, params={"format": output_format}).text

    def get_job_status(self, job_id: int) -> str:
        """Get the status of a job by id

        :type job_id: int
        :param job_id: The ID of the job

        :rtype: str
        :returns: The status of the job

        :raises TypeError: If name is not of type str
        """
        if not isinstance(job_id, int):
            raise TypeError(f"job_id must be of type int, but received {type(job_id)}")

        return self.get_job(job_id=job_id).get("status")



"""
Data schemas
"""

from typing import Union, Optional, Literal
import json
from dataclasses import dataclass


@dataclass
class _BaseSchema:
    """Base Dataclass Schema"""

    def dict(self) -> dict:
        """Get a dict representation of the Dataclass

        :rtype: dict
        :returns: A dict representation of the Dataclass
        """
        return self.__dict__

    def json(self, pretty=False) -> str:
        """Get a JSON representation of the Dataclass

        :type pretty: bool
        :param pretty: Get pretty JSON

        :rtype: str
        :returns: A JSON representation of the Dataclass
        """
        if pretty:
            indent = 4

        else:
            indent = None

        return json.dumps(self.dict(), indent=indent)

    @staticmethod
    def validate_string(name: str, value: str) -> None:
        """Validate a string

        :type value: str
        :param value: Value to validate
        :type name: str
        :param name: Name of the value
        """
        if not isinstance(value, str):
            raise TypeError(f"{name} must be a string, but received {type(value)}")

    @staticmethod
    def validate_boolean(name: str, value: bool) -> None:
        """Validate a boolean

        :type value: bool
        :param value: Value to validate
        :type name: str
        :param name: Name of the value
        """
        if not isinstance(value, bool):
            raise TypeError(f"{name} must be a boolean, but received {type(value)}")

    @staticmethod
    def validate_integer(name: str, value: int) -> None:
        """Validate an integer

        :type value: int
        :param value: Value to validate
        :type name: str
        :param name: Name of the value
        """
        if not isinstance(value, int):
            raise TypeError(f"{name} must be an integer, but received {type(value)}")


@dataclass
class InventoryRequestSchema(_BaseSchema):
    """Inventory Request Schema

    :type name: str
    :cvar name: Name of the Inventory
    :type description: str
    :cvar description: Description of the Inventory
    :type organization: str
    :cvar organization: Organization the Inventory belongs to
    :type kind: Optional[str] = ""
    :cvar kind: Kind of the Inventory
    :type host_filter: Optional[str] = ""
    :cvar host_filter: Host filter of the Inventory
    :type variables: Optional[Union[str, dict]] = "---"
    :cvar variables: Variables of the all group of the Inventory in JSON or YAML format
    :type prevent_instance_group_fallback: Optional[bool] = False
    :cvar prevent_instance_group_fallback: Whether to prevent fallback
    """

    name: str
    description: str
    organization: int
    kind: Optional[str] = ""
    host_filter: Optional[str] = None
    variables: Optional[Union[str, dict]] = "---"
    prevent_instance_group_fallback: Optional[bool] = False

    def __post_init__(self):
        if isinstance(self.variables, dict):
            self.variables = json.dumps(self.variables)

        self.validate_string("name", self.name)
        self.validate_string("description", self.description)
        self.validate_integer("organization", self.organization)
        self.validate_string("kind", self.kind)

        if self.host_filter:
            self.validate_string("host_filter", self.host_filter)

        self.validate_boolean("prevent_instance_group_fallback", self.prevent_instance_group_fallback)


@dataclass
class InventoryHostRequestSchema(_BaseSchema):
    """Inventory Host Request Schema

    :type name: str
    :cvar name: Name of the Host
    :type description: str
    :cvar description: Description of the Host
    :type enabled: Optional[bool] = True
    :cvar enabled: Whether to enable the Host
    :type instance_id: Optional[str] = ""
    :cvar instance_id: Instance ID of the Host
    :type variables: Optional[Union[str, dict]] = "---"
    :cvar variables: Variables of the host in JSON or YAML format
    """

    name: str
    description: str
    enabled: Optional[bool] = True
    instance_id: Optional[str] = ""
    variables: Optional[Union[str, dict]] = "---"

    def __post_init__(self):
        if isinstance(self.variables, dict):
            self.variables = json.dumps(self.variables)

        self.validate_string("name", self.name)
        self.validate_string("description", self.description)
        self.validate_boolean("enabled", self.enabled)
        self.validate_string("instance_id", self.instance_id)


@dataclass
class InventoryGroupRequestSchema(_BaseSchema):
    """Inventory Group Request Schema

    :type name: str
    :cvar name: Name of the Group
    :type description: str
    :cvar description: Description of the Group
    :type variables: Optional[Union[str, dict]] = "---"
    :cvar variables: Variables of the group in JSON or YAML format
    """

    name: str
    description: str
    variables: Optional[Union[str, dict]] = "---"

    def __post_init__(self):
        if isinstance(self.variables, dict):
            self.variables = json.dumps(self.variables)

        self.validate_string("name", self.name)
        self.validate_string("description", self.description)


@dataclass
class OrganizationRequestSchema(_BaseSchema):
    """Organization Request Schema

    :type name: str
    :cvar name: Name of the Group
    :type description: str
    :cvar description: Description of the Group
    :type max_hosts: Optional[int] = 0
    :cvar max_hosts: The maximum hosts 0 means unlimited
    :type default_environment: Optional[int] = None
    :cvar default_environment: The id of the execution environment
    """

    name: str
    description: str
    max_hosts: Optional[int] = 0
    default_environment: Optional[int] = None

    def __post_init__(self):
        self.validate_string("name", self.name)
        self.validate_string("description", self.description)
        self.validate_integer("max_hosts", self.max_hosts)

        if self.default_environment:
            self.validate_integer("default_environment", self.default_environment)


@dataclass
class JobTemplateRequestSchema(_BaseSchema):  # pylint: disable=too-many-instance-attributes
    """Job Template Request Schema"""

    name: Optional[str] = ""
    description: Optional[str] = ""
    job_type: Optional[Literal["run", "check"]] = "run"
    inventory: Optional[int] = None
    project: Optional[int] = None
    playbook: Optional[str] = ""
    scm_branch: Optional[str] = ""
    forks: Optional[int] = 0
    limit: Optional[str] = ""
    verbosity: Optional[Literal[0, 1, 2, 3, 4, 5]] = 0
    extra_vars: Optional[str] = ""
    job_tags: Optional[str] = ""
    force_handlers: Optional[bool] = False
    skip_tags: Optional[str] = ""
    start_at_task: Optional[str] = ""
    timeout: Optional[int] = 0
    use_fact_cache: Optional[bool] = False
    execution_environment: Optional[int] = None
    host_config_key: Optional[str] = ""
    ask_scm_branch_on_launch: Optional[bool] = False
    ask_diff_mode_on_launch: Optional[bool] = False
    ask_variables_on_launch: Optional[bool] = False
    ask_limit_on_launch: Optional[bool] = False
    ask_tags_on_launch: Optional[bool] = False
    ask_skip_tags_on_launch: Optional[bool] = False
    ask_job_type_on_launch: Optional[bool] = False
    ask_verbosity_on_launch: Optional[bool] = False
    ask_inventory_on_launch: Optional[bool] = False
    ask_credential_on_launch: Optional[bool] = False
    ask_execution_environment_on_launch: Optional[bool] = False
    ask_labels_on_launch: Optional[bool] = False
    ask_forks_on_launch: Optional[bool] = False
    ask_job_slice_count_on_launch: Optional[bool] = False
    ask_timeout_on_launch: Optional[bool] = False
    ask_instance_groups_on_launch: Optional[bool] = False
    survey_enabled: Optional[bool] = False
    become_enabled: Optional[bool] = False
    diff_mode: Optional[bool] = False
    allow_simultaneous: Optional[bool] = False
    job_slice_count: Optional[int] = 1
    webhook_service: Optional[Literal["", "github", "gitlab", "bitbucket_dc"]] = None
    webhook_credential: Optional[int] = None
    prevent_instance_group_fallback: Optional[bool] = False



import pytest
from ansible_aap_api_client import JobManagement


def test_run_job(requests_get_singe_job_template, requests_get_single_inventory, requests_job_template_job_launch):
    job_management = JobManagement(
        base_url="https://localhost:5000",
        username="test",
        password="test",
        ssl_verify=False,
        job_template_name="Demo Job Template",
        inventory_name="Demo Inventory",
    )

    job_management.run()


def test_run_job_bad():
    with pytest.raises(TypeError):
        job_management = JobManagement(
            base_url="https://localhost:5000",
            username="test",
            password="test",
            ssl_verify=False,
            job_template_name=1,
            inventory_name="Demo Inventory",
        )

    with pytest.raises(TypeError):
        job_management = JobManagement(
            base_url="https://localhost:5000",
            username="test",
            password="test",
            ssl_verify=False,
            job_template_name="Demo Job Template",
            inventory_name=1,
        )



"""
CLI for ansible_aap_api_client
"""

from argparse import ArgumentParser
from ansible_aap_api_client.job_management import JobManagement


def cli_argument_parser() -> ArgumentParser:
    """Function to create the argument parser

    :rtype: ArgumentParser
    :returns: The argument parser
    """
    arg_parser = ArgumentParser(description="ansible-aap-api-client-cli")

    arg_parser.add_argument("-b", "--base-url", required=True, help="Base URL for the Tower/AAP")
    arg_parser.add_argument("-u", "--username", required=True, help="The username")
    arg_parser.add_argument("-p", "--password", required=True, help="The password")

    subparsers = arg_parser.add_subparsers(
        title="commands",
        description="Valid commands: a single command is required",
        help="CLI Help",
        dest="a single command please see the -h option",
    )
    subparsers.required = True

    # This is the sub parser to jon a job template
    arg_parser_run_job_template = subparsers.add_parser("run-job-template", help="Run a job template in Tower/AAP")
    arg_parser_run_job_template.set_defaults(which_sub="run-job-template")
    arg_parser_run_job_template.add_argument(
        "-t", "--template-name", required=True, help="The name of the Job Template"
    )
    arg_parser_run_job_template.add_argument("-i", "--inventory-name", required=True, help="The name of Inventory")

    return arg_parser


def cli() -> None:  # pragma: no cover
    """Function to run the command line
    :rtype: None
    :returns: Nothing it is the CLI
    """
    arg_parser = None

    try:
        arg_parser = cli_argument_parser()
        args = arg_parser.parse_args()

        if args.which_sub == "run-job-template":
            job_mgmnt_obj = JobManagement(
                base_url=args.base_url,
                username=args.username,
                password=args.password,
                ssl_verify=False,
                job_template_name=args.template_name,
                inventory_name=args.inventory_name,
            )

            job_mgmnt_obj.poll_completion(print_status=True)

            print(job_mgmnt_obj.get_job_stdout(job_mgmnt_obj.job_id, "txt"))

    except AttributeError as error:
        print(f"\n !!! {error} !!! \n")
        arg_parser.print_help()

    except FileNotFoundError as error:
        print(f"\n !!! {error} !!! \n")
        arg_parser.print_help()

    except FileExistsError as error:
        print(f"\n !!! {error} !!! \n")
        arg_parser.print_help()

    except Exception as error:  # pylint: disable=broad-exception-caught
        print(f"\n !!! {error} !!! \n")
        arg_parser.print_help()



import pytest
from ansible_aap_api_client import Inventory, InventoryRequestSchema


def test_create_inventory(requests_create_inventory, inventory_request_schema):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    response = inventory.create_inventory(inventory=inventory_request_schema)


def test_create_inventory_bad(requests_create_inventory):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        inventory.create_inventory(inventory="bad")


def test_get_all_inventories(requests_get_two_inventory):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    response = inventory.get_all_inventories()


def test_get_inventory(requests_get_single_inventory):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    response = inventory.get_inventory(name="Demo Inventory")


def test_get_inventory_bad(requests_get_single_inventory):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        inventory.get_inventory(name=1)


def test_get_inventory_id(requests_get_single_inventory):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    response = inventory.get_inventory_id(name="Demo Inventory")

    assert response == 1


def test_get_inventory_id_bad(requests_get_two_inventory):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(ValueError):
        inventory.get_inventory_id(name="Demo Inventory")

    with pytest.raises(TypeError):
        inventory.get_inventory_id(name=1)


def test_add_host_to_inventory(requests_add_host_inventory, inventory_host_request_schema):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    response = inventory.add_host_to_inventory(inventory_id=38, host=inventory_host_request_schema)


def test_add_host_to_inventory_bad(requests_add_host_inventory, inventory_host_request_schema):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        inventory.add_host_to_inventory(inventory_id=38, host="bad")

    with pytest.raises(TypeError):
        inventory.add_host_to_inventory(inventory_id="bad", host=inventory_host_request_schema)


def test_add_group_to_inventory(requests_add_group_inventory, inventory_group_request_schema):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    response = inventory.add_group_to_inventory(inventory_id=38, group=inventory_group_request_schema)


def test_add_group_to_inventory_bad(requests_add_group_inventory, inventory_group_request_schema):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        inventory.add_group_to_inventory(inventory_id=38, group="bad")

    with pytest.raises(TypeError):
        inventory.add_group_to_inventory(inventory_id="bad", group=inventory_group_request_schema)


def test_delete_inventory(requests_delete_inventory):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    response = inventory.delete_inventory(inventory_id=2)


def test_delete_inventory_bad(requests_delete_inventory):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        inventory.delete_inventory(inventory_id="bad")


def test_update_inventory_bad(requests_delete_inventory):
    inventory = Inventory(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        inventory.update_inventory(inventory_id="bad", inventory="bad")

    with pytest.raises(TypeError):
        inventory.update_inventory(inventory_id=2, inventory="bad")



import pytest
from ansible_aap_api_client import Group


def test_get_all_groups(requests_get_two_group):
    obj = Group(base_url="https://localhost:5000", username="test", password="test")
    response = obj.get_all_groups()


def test_get_group(requests_get_two_group):
    obj = Group(base_url="https://localhost:5000", username="test", password="test")
    obj.get_group(name="test_group")


def test_get_group_bad(requests_get_two_group):
    obj = Group(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.get_group(name=1)


def test_get_group_id(requests_get_single_group):
    obj = Group(base_url="https://localhost:5000", username="test", password="test")
    response = obj.get_group_id(name="test_group")

    assert response == 2


def test_get_group_id_bad(requests_get_two_group):
    obj = Group(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(ValueError):
        obj.get_group_id(name="test_group")

    with pytest.raises(TypeError):
        obj.get_group_id(name=1)


def test_add_host_to_group(requests_add_host_group, inventory_host_request_schema):
    obj = Group(base_url="https://localhost:5000", username="test", password="test")
    response = obj.add_host_to_group(group_id=2, host=inventory_host_request_schema)


def test_add_host_to_group_bad(requests_add_host_group, inventory_host_request_schema):
    obj = Group(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.add_host_to_group(group_id=2, host="bad")

    with pytest.raises(TypeError):
        obj.add_host_to_group(group_id="bad", host=inventory_host_request_schema)


def test_delete_group(requests_delete_group):
    obj = Group(base_url="https://localhost:5000", username="test", password="test")
    response = obj.delete_group(group_id=2)


def test_delete_group_bad(requests_delete_group):
    obj = Group(base_url="https://localhost:5000", username="test", password="test")
    with pytest.raises(TypeError):
        obj.delete_group(group_id="bad")



"""
AAP Groups
"""

from ansible_aap_api_client.base_connection import _BaseConnection
from ansible_aap_api_client.schemas import InventoryHostRequestSchema, InventoryGroupRequestSchema


class Group(_BaseConnection):
    """Group class

    :type base_url: str
    :param base_url: The base url to use
    :type username: str
    :param username: The username to use
    :type password: str
    :param password: The password to use
    :type ssl_verify: Optional[Union[bool, str]] = True
    :param ssl_verify: The SSL verification True or False or a path to a certificate
    """

    groups_uri = "/groups/"

    def get_all_groups(self) -> dict:
        """Get all groups

        :rtype: Dict
        :returns: Response
        """
        return self._get(uri=self.groups_uri).json()

    def get_group(self, name: str) -> dict:
        """Get all instances of a group by name

        :type name: str
        :param name: The name of the group

        :rtype: Dict
        :returns: Response

        :raises TypeError: If name is not of type str
        """
        if not isinstance(name, str):
            raise TypeError(f"name must be of type str, but received {type(name)}")

        return self._get(uri=self.groups_uri, params={"name": name}).json()

    def get_group_id(self, name: str) -> int:
        """Get the id of a group if one exists

        :type name: str
        :param name: The name of the group

        :rtype: int
        :returns: The id of the group

        :raises ValueError: If zero or more than one instance is found
        :raises TypeError: If name is not of type str
        """
        if not isinstance(name, str):
            raise TypeError(f"name must be of type str, but received {type(name)}")

        response = self.get_group(name=name).get("results")

        if len(response) != 1:
            raise ValueError(f"found {len(response)} groups with name {name}")

        return response[0]["id"]

    def delete_group(self, group_id: int) -> int:
        """Delete group

        :type group_id: int
        :param group_id: The group id

        :rtype: Integer
        :returns: Response Status Code

        :raises TypeError: If inventory_id is not of type int
        """
        uri = f"{self.groups_uri}{group_id}/"

        if not isinstance(group_id, int):
            raise TypeError(f"group_id must be of type int, but received {type(group_id)}")

        return self._delete(uri=uri).status_code

    def update_group(self, group_id: int, group: InventoryGroupRequestSchema) -> dict:
        """Update group

        :type group_id: int
        :param group_id: The group id
        :type group: InventoryGroupRequestSchema
        :param group: The group data

        :rtype: Dict
        :returns: Response

        :raises TypeError: If inventory is not a InventoryRequestSchema
        :raises TypeError: If inventory_id is not of type int
        """
        uri = f"{self.groups_uri}{group_id}/"

        if not isinstance(group_id, int):
            raise TypeError(f"group_id must be of type int, but received {type(group_id)}")

        if not isinstance(group, InventoryGroupRequestSchema):
            raise TypeError(f"group must be of type InventoryGroupRequestSchema, but received {type(group)}")

        return self._patch(uri=uri, json_data=group.dict()).json()

    def add_host_to_group(self, group_id: int, host: InventoryHostRequestSchema) -> dict:
        """Add host to group

        :type group_id: int
        :param group_id: The group id
        :type host: InventoryHostRequestSchema
        :param host: The host to add

        :rtype: Dict
        :returns: Response

        :raises TypeError: If host is not of type InventoryHostRequestSchema
        """
        uri = f"{self.groups_uri}{group_id}/hosts/"

        if not isinstance(group_id, int):
            raise TypeError(f"group_id must be of type int, but received {type(group_id)}")

        if not isinstance(host, InventoryHostRequestSchema):
            raise TypeError(f"host must be of type InventoryHostRequestSchema, but received {type(host)}")

        return self._post(uri=uri, json_data=host.dict()).json()



"""
AAP API client for job templates
"""

from ansible_aap_api_client.base_connection import _BaseConnection


class JobTemplate(_BaseConnection):
    """Job Templates class

    :type base_url: str
    :param base_url: The base url to use
    :type username: str
    :param username: The username to use
    :type password: str
    :param password: The password to use
    :type ssl_verify: Optional[Union[bool, str]] = True
    :param ssl_verify: The SSL verification True or False or a path to a certificate
    """

    job_templates_uri = "/job_templates/"

    def get_all_job_templates(self) -> dict:
        """Get all job templates

        :rtype: Dict
        :returns: Response
        """
        return self._get(uri=self.job_templates_uri).json()

    def get_job_template(self, name: str) -> dict:
        """Get all instances of a job template by name

        :type name: str
        :param name: The name of the job template

        :rtype: Dict
        :returns: Response

        :raises TypeError: If name is not of type str
        """
        if not isinstance(name, str):
            raise TypeError(f"name must be of type str, but received {type(name)}")

        return self._get(uri=self.job_templates_uri, params={"name": name}).json()

    def get_job_template_id(self, name: str) -> int:
        """Get the id of a job template if one exists

        :type name: str
        :param name: The name of the job template

        :rtype: int
        :returns: The id of the job template

        :raises ValueError: If zero or more than one instance is found
        :raises TypeError: If name is not of type str
        """
        if not isinstance(name, str):
            raise TypeError(f"name must be of type str, but received {type(name)}")

        response = self.get_job_template(name=name).get("results")
        if len(response) != 1:
            raise ValueError(f"Expected 1 job template, but received {len(response)}")

        return response[0]["id"]

    def launch_job_template(self, job_template_id: int, **kwargs) -> dict:
        """Launch a job template

        :type job_template_id: int
        :param job_template_id: The id of the job template

        :param kwargs: The key word args to launch the job with, example: inventory=1, extra_vars={"key": "value"}

        :rtype: Dict
        :returns: Response

        :raises TypeError: If job_template_id is not of type int
        """
        uri = f"{self.job_templates_uri}{job_template_id}/launch/"

        if not isinstance(job_template_id, int):
            raise TypeError(f"job_template_id must be of type int, but received {type(job_template_id)}")

        return self._post(uri=uri, json_data=kwargs).json()
